SystemOrganization addCategory: #CodeCityLayouts!

ProtoObject subclass: #BoundingBox
	instanceVariableNames: 'origin dimension'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!BoundingBox commentStamp: '<historical>' prior: 0!
BoundingBox represents the minimum boundaries around one or more glyphs

Instance Variables:
	dimension	<Jun3dPoint>	size
	origin	<Jun3dPoint>	point of start

!

!BoundingBox class methodsFor: 'instance creation' stamp: 'StephaneDucasse 02/08/12 5:55'!
newOrigin: aCoordinate dimension: aDimension
	^self basicNew origin: aCoordinate; dimension: aDimension; yourself! !

!BoundingBox methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
dimension
	^dimension! !

!BoundingBox methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
dimension: anObject
	dimension := anObject! !

!BoundingBox methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
origin
	^origin! !

!BoundingBox methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
origin: anObject
	origin := anObject! !

ProtoObject subclass: #SpaceManager
	instanceVariableNames: ''
	classVariableNames: 'uniqueInstance'
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!SpaceManager commentStamp: '<historical>' prior: 0!
SpaceManager  is a helper class used to compute the boundaries of a collection of glyphs 
!

!SpaceManager class methodsFor: 'instance creation' stamp: 'StephaneDucasse 02/08/12 5:55'!
new
	^self error: 'Use uniqueInstance instead'! !

!SpaceManager class methodsFor: 'instance creation' stamp: 'StephaneDucasse 02/08/12 5:55'!
uniqueInstance
	^uniqueInstance isNil 
		ifTrue: [uniqueInstance := self basicNew initialize]
		ifFalse: [uniqueInstance]! !

!SpaceManager methodsFor: 'containment' stamp: 'StephaneDucasse 02/08/12 5:55'!
boundingBoxAroundGlyphs: aGlyphCollection 
	^aGlyphCollection isEmpty 
		ifTrue: [BoundingBox newOrigin: 0 @ 0 dimension: 0 @ 0]
		ifFalse: 
			[| origin extension |
			origin := aGlyphCollection first position.
			extension := aGlyphCollection first position 
						+ aGlyphCollection first dimension.
			aGlyphCollection do: 
					[:each | 
					origin := origin min: each position.
					extension := extension max: each position + each dimension].
			BoundingBox newOrigin: origin dimension: extension - origin]! !

!SpaceManager methodsFor: 'initialize-release' stamp: 'StephaneDucasse 02/08/12 5:55'!
initialize
	super initialize.
	^self! !

Object subclass: #AbstractLayout
	instanceVariableNames: 'horizontalMargin horizontalGap verticalMargin verticalGap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!AbstractLayout commentStamp: '<historical>' prior: 0!
AbstractLayout is where the common behavior of the layouts stays.

Subclasses must implement the following messages:
	layout
		layoutGlyphs:origin:

Instance Variables:
	horizontalGap	<Number>	horizontal space between glyphs
	horizontalMargin	<Number>	horizontal space between marginal glyphs and internal boundary of the container glyph
	verticalGap	<Number>	vertical space between glyphs
	verticalMargin	<Number>	vertical space between marginal glyphs and internal boundary of the container glyph (bumper)

!

!AbstractLayout class methodsFor: 'defaults' stamp: 'StephaneDucasse 02/08/12 5:55'!
defaultHorizontalGap
	^1! !

!AbstractLayout class methodsFor: 'defaults' stamp: 'StephaneDucasse 02/08/12 5:55'!
defaultHorizontalMargin
	^0.5! !

!AbstractLayout class methodsFor: 'defaults' stamp: 'StephaneDucasse 02/08/12 5:55'!
defaultVerticalGap
	^1! !

!AbstractLayout class methodsFor: 'defaults' stamp: 'StephaneDucasse 02/08/12 5:55'!
defaultVerticalMargin
	^0.2! !

!AbstractLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^self subclassResponsibility! !

!AbstractLayout class methodsFor: 'instance creation' stamp: 'StephaneDucasse 02/08/12 5:55'!
new
	^self basicNew initialize! !

!AbstractLayout class methodsFor: 'instance creation' stamp: 'StephaneDucasse 02/08/12 5:55'!
newHorizontalMargin: hMargin horizontalGap: hGap verticalMargin: vMargin verticalGap: vGap 
	^self new 
		horizontalMargin: hMargin;
		horizontalGap: hGap;
		verticalMargin: vMargin;
		verticalGap: vGap;
		yourself! !

!AbstractLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
properties
	^OrderedCollection 
		with: #horizontalMargin 
		with: #horizontalGap 
		with: #verticalMargin
		with: #verticalGap! !

!AbstractLayout methodsFor: 'comparing' stamp: 'StephaneDucasse 02/08/12 5:55'!
= aLayout 
	^self class = aLayout class 
		ifTrue: 
			[horizontalGap = aLayout horizontalGap 
				& (verticalGap = aLayout verticalGap) 
					& (horizontalMargin = aLayout horizontalMargin) 
					& (verticalMargin = aLayout verticalMargin)]
		ifFalse: [false]! !

!AbstractLayout methodsFor: 'printing' stamp: 'StephaneDucasse 02/08/12 5:55'!
asScript
	| stream |
	stream := String new writeStream.
	stream 
		nextPutAll: self class printString; cr;
		tab; nextPutAll: 'newHorizontalMargin: ', self horizontalMargin printString; cr;
		tab; nextPutAll: 'horizontalGap: ', self horizontalGap printString; cr;
		tab; nextPutAll: 'verticalMargin: ', self verticalMargin printString; cr;
		tab; nextPutAll: 'verticalGap: ', self verticalGap printString.
	^stream contents! !

!AbstractLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
bumperElevate: aGlyph 
	aGlyph compositePosition: aGlyph position + (0 , 0 , self verticalMargin)! !

!AbstractLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
computeLargestDimensionForGlyphs: glyphs
	^Jun3dPoint 	x: (glyphs inject: 0 into: [:maxX :each | maxX max: each dimension x])
				y: (glyphs inject: 0 into: [:maxY :each | maxY max: each dimension y])
				z: (glyphs inject: 0 into: [:maxZ :each | maxZ max: each dimension z])! !

!AbstractLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
diameterOfCircumcircleForBoxSized: a2dDimension 
	^(a2dDimension x squared + a2dDimension y squared) sqrt! !

!AbstractLayout methodsFor: 'comparing' stamp: 'StephaneDucasse 02/08/12 5:55'!
hash
	
	^((horizontalGap hash bitXor: verticalGap) bitXor: horizontalMargin hash)
		bitXor: verticalMargin hash! !

!AbstractLayout methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
horizontalGap
	^horizontalGap! !

!AbstractLayout methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
horizontalGap: anObject
	horizontalGap := anObject! !

!AbstractLayout methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
horizontalMargin
	^horizontalMargin! !

!AbstractLayout methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
horizontalMargin: anObject
	horizontalMargin := anObject! !

!AbstractLayout methodsFor: 'initialize-release' stamp: 'StephaneDucasse 02/08/12 5:55'!
initialize
	super initialize.
	self horizontalMargin:  self class defaultHorizontalMargin.
	self horizontalGap: self class defaultHorizontalGap.
	self verticalMargin: self class defaultVerticalMargin.
	self verticalGap: self class defaultVerticalGap! !

!AbstractLayout methodsFor: 'initialize-release' stamp: 'StephaneDucasse 02/08/12 5:55'!
initializeHorizontalMargin: hMar horizontalGap: hGap verticalMargin: vMar verticalGap: vGap
	super initialize.
	horizontalMargin := hMar.
	horizontalGap := hGap.
	verticalMargin := vMar.
	verticalGap := vGap.
	^self! !

!AbstractLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: originPosition
	^self subclassResponsibility! !

!AbstractLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
moveToOriginSceneWithRadius: containerRadius containingFigures: nodeFigures 
	nodeFigures do: 
			[:each | 
			each compositePosition: each position + (containerRadius , containerRadius , 0)]! !

!AbstractLayout methodsFor: 'printing' stamp: 'StephaneDucasse 02/08/12 5:55'!
printOn: aStream 
	aStream nextPutAll: self class printString;
		nextPutAll: ' <hMargin:', self horizontalMargin printString;
		nextPutAll: ',hGap:', self horizontalGap printString;
		nextPutAll: ',vMargin:', self verticalMargin printString;
		nextPutAll: ',vGap:',  self verticalGap printString;
		nextPut:$>! !

!AbstractLayout methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
verticalGap
	^verticalGap! !

!AbstractLayout methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
verticalGap: anObject
	verticalGap := anObject! !

!AbstractLayout methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
verticalMargin
	^verticalMargin! !

!AbstractLayout methodsFor: 'accessing' stamp: 'StephaneDucasse 02/08/12 5:55'!
verticalMargin: anObject
	verticalMargin := anObject! !

AbstractLayout subclass: #BricksLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!BricksLayout commentStamp: '<historical>' prior: 0!
CuboidLayout is a 3d layout where the glyphs are placed in vertical layers of maximal 4 glyphs!

!BricksLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^#bricks! !

!BricksLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: origin3d 
	| rowCounter columnCounter floorCounter numberOfElementsInARow maxDimension margin3d gap3d counter3d capacity3d|
	numberOfElementsInARow := 2.
	rowCounter := 0.
	columnCounter := 0.
	floorCounter := 0.
	margin3d := horizontalMargin , horizontalMargin , verticalMargin.
	gap3d := horizontalGap, horizontalGap, verticalGap.
	maxDimension := self computeLargestDimensionForGlyphs: aCollectionOfGlyphs.
	aCollectionOfGlyphs do: 
			[:each | 
			| offsetWithinCell3d offsetToCell3d |
			offsetWithinCell3d := (maxDimension - each dimension) / 2.
			counter3d := rowCounter , columnCounter , floorCounter.
			offsetToCell3d := counter3d * (maxDimension + gap3d).
			each compositePosition: origin3d + margin3d + offsetToCell3d + offsetWithinCell3d.
			rowCounter >= (numberOfElementsInARow - 1) 
				ifTrue: 
					[rowCounter := 0.
					columnCounter >= (numberOfElementsInARow - 1) 
						ifTrue: 
							[columnCounter := 0.
							floorCounter := floorCounter + 1]
						ifFalse: [columnCounter := columnCounter + 1]]
				ifFalse: [rowCounter := rowCounter + 1]].
	capacity3d := numberOfElementsInARow, numberOfElementsInARow, (floorCounter + 1).
	^margin3d + (capacity3d * maxDimension) + ((capacity3d - 1) * gap3d) + margin3d! !

AbstractLayout subclass: #ChessboardLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!ChessboardLayout commentStamp: '<historical>' prior: 0!
ChessboardLayout is a simple layout, where the space is split into rectangles with the size large enought to host any of the glyphs. Each glyph is then placed in the center of such a cell, on the floor (z =bumper)!

!ChessboardLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^#chessboard! !

!ChessboardLayout methodsFor: 'private' stamp: 'StephaneDucasse 02/08/12 5:55'!
capacityDistributionForCardinality: aNumber
	^Jun3dPoint 	x: aNumber sqrt ceiling
					y: aNumber sqrt ceiling
					z: 1! !

!ChessboardLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: originPosition 
	"lays out the components and answers the new dimension of the container"

	| maxDimension distribution sizeX sizeY sizeZ |
	maxDimension := self computeLargestDimensionForGlyphs: aCollectionOfGlyphs.
	distribution := self 
				capacityDistributionForCardinality: aCollectionOfGlyphs size.
	aCollectionOfGlyphs do: 
			[:each | 
			| row column index newX newY newZ |
			index := aCollectionOfGlyphs indexOf: each.
			row := (index - 1) // distribution x.
			column := (index - 1) \\ distribution y.
			newX := originPosition x + horizontalMargin 
						+ (row * (maxDimension x + horizontalGap)) 
							+ ((maxDimension x - each dimension x) / 2).
			newY := originPosition y + horizontalMargin 
						+ (column * (maxDimension y + horizontalGap)) 
							+ ((maxDimension y - each dimension y) / 2).
			newZ := originPosition z + verticalMargin.
			each compositePosition: newX , newY , newZ].
	sizeX := distribution x * maxDimension x 
				+ ((distribution x - 1) * horizontalGap) + (horizontalMargin * 2).
	sizeY := distribution y * maxDimension y 
				+ ((distribution y - 1) * horizontalGap) + (horizontalMargin * 2).
	sizeZ := maxDimension z + (verticalMargin * 2).
	^sizeX , sizeY , sizeZ! !

AbstractLayout subclass: #CircleLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!CircleLayout commentStamp: '<historical>' prior: 0!
CircleLayout is a layout where the glyphs are placed in a circle without overlapping!

!CircleLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^#circle! !

!CircleLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: originPosition
	| angleTemp angle radius center largestSize gap |
	largestSize := self computeLargestDimensionForGlyphs: aCollectionOfGlyphs.
	gap := self diameterOfCircumcircleForBoxSized: largestSize.
	angleTemp := 2 * Float pi / aCollectionOfGlyphs size.
	aCollectionOfGlyphs size = 1 
		ifTrue: [radius := 0]
		ifFalse: [radius := (gap squared / (2 * (1 - angleTemp cos))) sqrt].
	center := Point x: originPosition x + radius + (largestSize x / 2) + self horizontalMargin
				   y: originPosition y + radius + (largestSize y / 2) + self horizontalMargin.
	angle := 0.
	aCollectionOfGlyphs do: 
			[:each | 
			| computedPosition |
			computedPosition := center + (Point r: radius theta: angle) 
						- ((each dimension x , each dimension y , 0) / 2).
			each compositePosition: computedPosition x, computedPosition y, (originPosition z + self verticalMargin).
			self bumperElevate: each.
			angle := angle + angleTemp].
	self todo: 'to the extreme, this will become an elipse: try with (10,1)'.
	^(2 * radius + largestSize x + (self horizontalMargin * 2)), 
	 (2 * radius + largestSize y + (self horizontalMargin * 2)), 
	 (largestSize z + (self verticalMargin * 2))! !

AbstractLayout subclass: #ConcentricalLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!ConcentricalLayout commentStamp: '<historical>' prior: 0!
TargetLayout lays out glyphs in concentrical circles starting in the center.

!

!ConcentricalLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^#concentrical! !

!ConcentricalLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: originPosition 
	| largestDiameter angleIncrement angle radius center noOfCircles concentricGap containerRadius largestDimension |
	largestDimension := self computeLargestDimensionForGlyphs: aCollectionOfGlyphs.
	largestDiameter := self diameterOfCircumcircleForBoxSized: largestDimension.
	concentricGap := largestDiameter * 2.
	noOfCircles := 0.
	center := Point x: originPosition x + self horizontalMargin y: originPosition y + self horizontalMargin.
	radius := 0.
	angle := 2 * Float pi.
	angleIncrement := 1.
	aCollectionOfGlyphs do: [:each | | computedPosition |
			angle + angleIncrement > (2 * Float pi) 
				ifTrue: 
					[radius := radius + concentricGap.
					noOfCircles := noOfCircles + 1.
					angle := 0.
					angleIncrement := ((2 * radius squared - largestDiameter squared) 
								/ (2 * (radius ** 2))) arcCos].
			computedPosition := center + (Point r: radius theta: angle) 
						- ((each dimension x , each dimension y , 0) / 2).
			each compositePosition: computedPosition x, computedPosition y, (originPosition z + self verticalMargin).
			self bumperElevate: each.
			angle := angle + angleIncrement].
	containerRadius := noOfCircles * concentricGap + (largestDiameter / 2) 
				+ (2 * self verticalMargin).
	self moveToOriginSceneWithRadius: containerRadius
		containingFigures: aCollectionOfGlyphs.
	^(2 * (containerRadius + self horizontalMargin)),
	 (2 * (containerRadius + self horizontalMargin)),
	 (largestDimension z + (self verticalMargin * 2))! !

AbstractLayout subclass: #LineLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!LineLayout commentStamp: '<historical>' prior: 0!
LineLayout aligns the glyphs on one single line along the X axis.!

!LineLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^#line! !

!LineLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: originPosition 
	"lays out the components and answers the new dimension of the container"

	| maxDimension sizeX sizeY sizeZ |
	maxDimension := self computeLargestDimensionForGlyphs: aCollectionOfGlyphs.
	aCollectionOfGlyphs do: 
			[:each | 
			| row column index newX newY newZ |
			index := aCollectionOfGlyphs indexOf: each.
			row := 0.
			column := index - 1.
			newX := originPosition x + horizontalMargin 
						+ (column * (maxDimension x + horizontalGap)) 
							+ ((maxDimension x - each dimension x) / 2).
			newY := originPosition y + horizontalMargin 
						+ (row * (maxDimension y + horizontalGap)) 
							+ ((maxDimension y - each dimension y) / 2).
			newZ := originPosition z + verticalMargin.
			each compositePosition: newX , newY , newZ].	
	sizeX := aCollectionOfGlyphs size * maxDimension x 
				+ ((aCollectionOfGlyphs size - 1) * horizontalGap) + (horizontalMargin * 2).
	sizeY := maxDimension y + (horizontalMargin * 2).
	sizeZ := maxDimension z + (verticalMargin * 2).
	^sizeX , sizeY , sizeZ! !

AbstractLayout subclass: #ProgressiveBricksLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!ProgressiveBricksLayout commentStamp: '<historical>' prior: 0!
ProgressiveBricksLayout is similar to bricks, but the number of elements on one level depends on the total number of elements. This addresses the problem of extremely high buildings and of the constant width of the buildings in the regular BricksLayout
!

!ProgressiveBricksLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^#'progressive bricks'! !

!ProgressiveBricksLayout methodsFor: 'coordinate computation' stamp: 'StephaneDucasse 02/08/12 5:55'!
brickIndexWithinLevelOfBrickWithIndex: index forSideCapacity: sideCapacity 
	| levelCapacity |
	levelCapacity := self 
				levelCapacityForSideCapacity: sideCapacity.
	^index  \\ levelCapacity! !

!ProgressiveBricksLayout methodsFor: 'coordinate computation' stamp: 'StephaneDucasse 02/08/12 5:55'!
brickIndexWithinSideOfBrickWithIndex: index forSideCapacity: sideCapacity 
	^sideCapacity == 1 
		ifTrue: [0]
		ifFalse: 
			[| brickIndexWithinLayer |
			brickIndexWithinLayer := self brickIndexWithinLevelOfBrickWithIndex: index
						forSideCapacity: sideCapacity.
			brickIndexWithinLayer \\ (sideCapacity - 1)]! !

!ProgressiveBricksLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
coordinateFactorOfBrickIndex: oneBasedIndex forSideCapacity: sideCapacity 
	| xFactor yFactor zFactor brickIndexWithinSide xValues yValues sideIndex index |
	index := oneBasedIndex - 1.
	"from here on, it's 0-indexed, for simpler algorithm description"
	zFactor := self levelIndexOfBrickIndex: index
				forSideCapacity: sideCapacity.
	brickIndexWithinSide := self brickIndexWithinSideOfBrickWithIndex: index
				forSideCapacity: sideCapacity.
	xValues := OrderedCollection 
				with: brickIndexWithinSide
				with: sideCapacity - 1
				with: sideCapacity - brickIndexWithinSide - 1
				with: 0.
	yValues := OrderedCollection
				with: 0
				with: brickIndexWithinSide
				with: sideCapacity - 1
				with: sideCapacity - brickIndexWithinSide - 1.
	sideIndex := self sideIndexOfBrickIndex: index
				forSideCapacity: sideCapacity.
	xFactor := xValues at: sideIndex + 1.
	yFactor := yValues at: sideIndex + 1.
	^xFactor , yFactor , zFactor! !

!ProgressiveBricksLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: origin3d 
	| bricksPerSide margin3d gap3d maxDimension containerSize numberOfLayers elementDistribution |
	bricksPerSide := self sideCapacityForCardinality: aCollectionOfGlyphs size.
	margin3d := horizontalMargin , horizontalMargin , verticalMargin.
	gap3d := horizontalGap , horizontalGap , verticalGap.
	maxDimension := self computeLargestDimensionForGlyphs: aCollectionOfGlyphs.
	1 to: aCollectionOfGlyphs size do: 	"still 1-indexed"
			[:each | 
			| glyph offsetWithinCell3d offsetToCell3d coordinateFactor3d |
			glyph := aCollectionOfGlyphs at: each.
			coordinateFactor3d := self coordinateFactorOfBrickIndex: each forSideCapacity: bricksPerSide.
			offsetToCell3d := coordinateFactor3d * (maxDimension + gap3d).
			offsetWithinCell3d := (maxDimension - glyph dimension) / 2.
			glyph compositePosition: origin3d + margin3d + offsetToCell3d + offsetWithinCell3d.
	].
	numberOfLayers := self levelIndexOfBrickIndex: aCollectionOfGlyphs size forSideCapacity: bricksPerSide.
	elementDistribution := bricksPerSide, bricksPerSide, numberOfLayers.
	containerSize := (margin3d * 2) + (elementDistribution * maxDimension) + ((elementDistribution - 1) * gap3d).
	^containerSize! !

!ProgressiveBricksLayout methodsFor: 'spatial properties' stamp: 'StephaneDucasse 02/08/12 5:55'!
levelCapacityForSideCapacity: sideCapacity 
	^sideCapacity == 1 ifTrue: [1] ifFalse: [4 * (sideCapacity - 1)]! !

!ProgressiveBricksLayout methodsFor: 'coordinate computation' stamp: 'StephaneDucasse 02/08/12 5:55'!
levelIndexOfBrickIndex: index forSideCapacity: sideCapacity 
	| levelCapacity |
	levelCapacity := self 
				levelCapacityForSideCapacity: sideCapacity.
	^index // levelCapacity! !

!ProgressiveBricksLayout methodsFor: 'spatial properties' stamp: 'StephaneDucasse 02/08/12 5:55'!
maximumBuildingCapacityForSideCapacity: sideCapacity
	|nextMinimum|
	nextMinimum :=  self minimumBuildingCapacityForSideCapacity: sideCapacity + 1.
	^nextMinimum - 1! !

!ProgressiveBricksLayout methodsFor: 'spatial properties' stamp: 'StephaneDucasse 02/08/12 5:55'!
minimumBuildingCapacityForSideCapacity: sideCapacity
	| levelCapacity minimumNumberOfLayers |
	levelCapacity := self levelCapacityForSideCapacity: sideCapacity.
	minimumNumberOfLayers := self minimumNumberOfLevelsForSideCapacity: sideCapacity.
	^minimumNumberOfLayers * levelCapacity! !

!ProgressiveBricksLayout methodsFor: 'spatial properties' stamp: 'StephaneDucasse 02/08/12 5:55'!
minimumNumberOfLevelsForSideCapacity: sideCapacity
	^(sideCapacity - 1) * 2! !

!ProgressiveBricksLayout methodsFor: 'spatial properties' stamp: 'StephaneDucasse 02/08/12 5:55'!
sideCapacityForCardinality: numberOfElements
	
	| sideCapacity |
	numberOfElements < 0
		ifTrue: [self error: 'set size should be a positive number'].
	sideCapacity := 0.
	[sideCapacity := sideCapacity + 1.
	(self maximumBuildingCapacityForSideCapacity: sideCapacity) >= numberOfElements] whileFalse.
	^sideCapacity! !

!ProgressiveBricksLayout methodsFor: 'coordinate computation' stamp: 'StephaneDucasse 02/08/12 5:55'!
sideIndexOfBrickIndex: index forSideCapacity: sideCapacity 
	^sideCapacity == 1 
		ifTrue: [0]
		ifFalse: 
			[| brickIndexWithinLayer |
			brickIndexWithinLayer := self brickIndexWithinLevelOfBrickWithIndex: index
						forSideCapacity: sideCapacity.
			brickIndexWithinLayer // (sideCapacity - 1)]! !

AbstractLayout subclass: #QuadraticLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!QuadraticLayout commentStamp: '<historical>' prior: 0!
QuadraticLayout is a variant of ChessboardLayout that optimize the space usage on only one of the two horizontal axes!

!QuadraticLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^#quadratic! !

!QuadraticLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: originPosition 
	| numberOfElementsInARow xCoord yCoord elementIndexInRow heightOfTallestInRow xOffset yOffset currentRowSize largestRowWidth totalHeight largestDimension |
	largestDimension := self computeLargestDimensionForGlyphs: aCollectionOfGlyphs.
	numberOfElementsInARow := aCollectionOfGlyphs size sqrt rounded.
	xCoord := originPosition x + horizontalMargin.
	yCoord := originPosition y + horizontalMargin.
	elementIndexInRow := 1.
	heightOfTallestInRow := 0.
	xOffset := 0.
	currentRowSize := horizontalMargin.
	largestRowWidth := 0.
	totalHeight := horizontalMargin.
	aCollectionOfGlyphs do: 
			[:each | 
			elementIndexInRow <= numberOfElementsInARow 
				ifTrue: 
					[xCoord := xCoord + xOffset.
					heightOfTallestInRow := heightOfTallestInRow max: each dimension y]
				ifFalse: 
					[currentRowSize := currentRowSize - horizontalGap + horizontalMargin.
					xCoord := originPosition x + horizontalMargin.
					yOffset := heightOfTallestInRow + horizontalGap.
					yCoord := yCoord + yOffset.
					largestRowWidth := largestRowWidth max: currentRowSize.
					currentRowSize := horizontalMargin.
					totalHeight := totalHeight + yOffset.
					heightOfTallestInRow := each dimension y.
					elementIndexInRow := 1].
			each compositePosition: xCoord , yCoord , (verticalMargin + originPosition z).
			xOffset := each dimension x + horizontalGap.
			currentRowSize := currentRowSize + xOffset.
			elementIndexInRow := elementIndexInRow + 1].
	totalHeight := totalHeight + heightOfTallestInRow + horizontalMargin.
	largestRowWidth := largestRowWidth max: currentRowSize.
	^largestRowWidth , totalHeight , (largestDimension z + (verticalMargin * 2))! !

AbstractLayout subclass: #SpiralLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityLayouts'!

!SpiralLayout commentStamp: '<historical>' prior: 0!
SpiralLayout lays out glyphs in a spiral starting from the center
!

!SpiralLayout class methodsFor: 'codecity' stamp: 'StephaneDucasse 02/08/12 5:55'!
displayableName
	^#spiral! !

!SpiralLayout methodsFor: 'private' stamp: 'StephaneDucasse 02/08/12 5:55'!
angleInCircleWithRadius: radius forChord: largestSize 
	^((2 * (radius ** 2) - (largestSize ** 2)) / (2 * (radius ** 2))) arcCos! !

!SpiralLayout methodsFor: 'private' stamp: 'StephaneDucasse 02/08/12 5:55'!
capacityForSize: size inCircleWithRadius: radius
	|angle|
	"answers the number of glyphs - float - that can be layed out on the circle perimeter"
	angle := self angleInCircleWithRadius: radius forChord: size.
	^2 * Float pi / angle! !

!SpiralLayout methodsFor: 'private' stamp: 'StephaneDucasse 02/08/12 5:55'!
currentSpireCapacityRadius: radius forElementsSized: largestSize 
	| currentSpireCapacity currentCircleCapacity nextCircleCapacity |
	currentCircleCapacity := self capacityForSize: largestSize
				inCircleWithRadius: radius.	"compute the aproximate circle capacity"
	nextCircleCapacity := self capacityForSize: largestSize
				inCircleWithRadius: radius.
	currentSpireCapacity := ((currentCircleCapacity + nextCircleCapacity) / 2) 
				floor.
	^currentSpireCapacity! !

!SpiralLayout methodsFor: 'layout' stamp: 'StephaneDucasse 02/08/12 5:55'!
layoutGlyphs: aCollectionOfGlyphs origin: originPosition
	| largestDiameter angleIncrement angle radius center concentricGap containerRadius largestDimension |
	largestDimension := self computeLargestDimensionForGlyphs: aCollectionOfGlyphs.
	largestDiameter := self diameterOfCircumcircleForBoxSized: largestDimension.
	center := Point x: originPosition x y: originPosition y.
	radius := largestDiameter.
	concentricGap := largestDiameter * 2.
	angle := 0.
	angleIncrement := 0.
	aCollectionOfGlyphs do: 
			[:each | | computedPosition radiusStepIncrement currentSpireCapacity |
			currentSpireCapacity := self currentSpireCapacityRadius: radius
						forElementsSized: largestDiameter.
			radiusStepIncrement := concentricGap / currentSpireCapacity.
			computedPosition := center + (Point r: radius theta: angle) 
						- ((each dimension x , each dimension y , 0) / 2).
			each compositePosition: computedPosition x, computedPosition y, (originPosition z + self verticalMargin).
			self bumperElevate: each.
			angleIncrement := self angleInCircleWithRadius: radius
						forChord: largestDiameter.
			angle := angle + angleIncrement.
			radius := radius + radiusStepIncrement].
	containerRadius := radius + (largestDiameter / 2) + (2 * self verticalMargin).
	self moveToOriginSceneWithRadius: containerRadius
		containingFigures: aCollectionOfGlyphs.
	^2 * containerRadius, 
	(2 * containerRadius), 
	(largestDimension z + (self verticalMargin * 2))! !
